@@ -1,5 +1,5 @@
 ##############################################################################
-#    Copyright (c) 2012-2016 Russell V. Lenth                                #
+#    Copyright (c) 2012-2024 Russell V. Lenth                                #
 #                                                                            #
 #    This file is part of the emmeans package for R (*emmeans*)              #
 #                                                                            #
@@ -20,6 +20,7 @@
 ##############################################################################
 
 # rbind method for emmGrid objects
+# ... and some other methods
 
 #' Combine or subset \code{emmGrid} objects
 #'
@@ -29,10 +30,8 @@
 #' doing this would be to obtain multiplicity-adjusted results for smaller
 #' or larger families of tests or confidence intervals. 
 #' 
-#' @param ... Additional arguments:
-#'   In \code{rbind}, object(s) of class \code{emmGrid}. 
-#'   In \code{"["}, it is ignored. 
-#'   In \code{subset}, it is passed to \code{[.emmGrid]}
+#' @param ... In \code{rbind}, object(s) of class \code{emmGrid}.
+#'   In others, additional arguments passed to other methods
 #' @param deparse.level (required but not used)
 #' @param adjust Character value passed to \code{\link{update.emmGrid}}
 #' 
@@ -40,6 +39,10 @@
 #'   the objects' coefficients, covariance structures, etc. But they 
 #'   are allowed to have different factors; a missing level \code{'.'}
 #'   is added to factors as needed.
+#' 
+#' @note These functions generally reset \code{by.vars} to \code{NULL};
+#' so if you want to keep any \dQuote{by} variables, you should follow-up
+#' with \code{\link{update.emmGrid}}.
 #'
 #' @return A revised object of class \code{emmGrid}
 #' @order 1
@@ -107,8 +110,7 @@ rbind.emmGrid = function(..., deparse.level = 1, adjust = "bonferroni") {
 }
 #' @rdname rbind.emmGrid
 #' @order 4
-#' @param e1 An \code{emmGrid} object
-#' @param e2 Another \code{emmGrid} object
+#' @param e1,e2,x,object Objects of class \code{emmGrid}
 #' @return The result of \code{e1 + e2} is the same as \code{rbind(e1, e2)}
 #' @method + emmGrid
 #' @export
@@ -123,7 +125,6 @@ rbind.emmGrid = function(..., deparse.level = 1, adjust = "bonferroni") {
 # if drop = TRUE, the levels of factors are reduced
 #' @rdname rbind.emmGrid
 #' @order 5
-#' @param x An \code{emmGrid} object to be subsetted
 #' @param i Integer vector of indexes
 #' @param drop.levels Logical value. If \code{TRUE}, the \code{"levels"} slot in
 #'   the returned object is updated to hold only the predictor levels that actually occur
@@ -152,7 +153,7 @@ rbind.emmGrid = function(..., deparse.level = 1, adjust = "bonferroni") {
 }
 
 #' @rdname rbind.emmGrid
-#' @order 7
+#' @order 9
 #' @param subset logical expression indicating which rows of the grid to keep
 #' @method subset emmGrid
 #' @export
@@ -166,3 +167,27 @@ subset.emmGrid = function(x, subset, ...) {
     x[sel, ...]
 }
 
+#' @rdname rbind.emmGrid
+#' @order 6
+#' @param n integer number of entries to include (or exclude if negative)
+#' @importFrom utils head
+#' @method head emmGrid
+#' @export
+head.emmGrid = function(x, n = 6, ...) {
+    s = sign(n[1])
+    n = min(abs(n[1]), nrow(x@grid))
+    x[s * seq_len(n)]
+}
+
+#' @rdname rbind.emmGrid
+#' @order 7
+#' @importFrom utils tail
+#' @method tail emmGrid
+#' @export
+tail.emmGrid = function(x, n = 6, ...) {
+    s = sign(n[1])
+    N = nrow(x@grid)
+    n = min(abs(n[1]), N)
+    x[s * (seq_len(n) - n + N)]
+}
+
